import Vapor
import HTTP

final class AcronymsController {

  func addRoutes(drop: Droplet) {
    let basic = drop.grouped("acronyms")
    basic.get(handler: index)
    basic.post(handler: create)
    basic.get(Acronym.self, handler: show)
    basic.patch(Acronym.self, handler: update)
    basic.delete(Acronym.self, handler: delete)
    basic.get(Acronym.self, "user", handler: userShow)
  }

  func index(request: Request) throws -> ResponseRepresentable {
    return try JSON(node: Acronym.all().makeNode())
  }
  
  func create(request: Request) throws -> ResponseRepresentable {
    var acronym = try request.acronym()
    try acronym.save()
    return acronym
  }
  
  func show(request: Request, acronym: Acronym) throws -> ResponseRepresentable {
    return acronym
  }
  
  func update(request: Request, acronym: Acronym) throws -> ResponseRepresentable {
    let new = try request.acronym()
    var acronym = acronym
    acronym.short = new.short
    acronym.long = new.long
    try acronym.save()
    return acronym
  }
  
  func delete(request: Request, acronym: Acronym) throws -> ResponseRepresentable {
    try acronym.delete()
    return JSON([:])
  }
  
  func userShow(request: Request, acronym: Acronym) throws -> ResponseRepresentable {
    let tiluser = try acronym.tiluser()
    return try JSON(node: tiluser?.makeNode())
  }

}

extension Request {
  func acronym() throws -> Acronym {
    guard let json = json else { throw Abort.badRequest }
    return try Acronym(node: json)
  }
}
