import Vapor
import HTTP
import Fluent

final class TILUserController {

  func addRoutes(drop: Droplet) {
    let basic = drop.grouped("users")
    basic.get(handler: index)
    basic.post(handler: create)
    basic.delete(TILUser.self, handler: delete)
    basic.get(TILUser.self, "acronyms", handler: acronymsIndex)
    basic.post(TILUser.self, "join", Group.self, handler: joinGroup)
    basic.get(TILUser.self, "groups", handler: groupsIndex)
  }

  func index(request: Request) throws -> ResponseRepresentable {
    return try JSON(node: TILUser.all().makeNode())
  }
  
  func create(request: Request) throws -> ResponseRepresentable {
    var tiluser = try request.tiluser()
    try tiluser.save()
    return tiluser
  }
  
  func delete(request: Request, tiluser: TILUser) throws -> ResponseRepresentable {
    try tiluser.delete()
    return JSON([:])
  }
  
  func acronymsIndex(request: Request, tiluser: TILUser) throws -> ResponseRepresentable {
    let children = try tiluser.acronyms()
    return try JSON(node: children.makeNode())
  }
  
  func joinGroup(request: Request, tiluser: TILUser, group: Group) throws -> ResponseRepresentable {
    var pivot = Pivot<TILUser, Group>(tiluser, group)
    try pivot.save()
    return tiluser
  }
  
  func groupsIndex(request: Request, tiluser: TILUser) throws -> ResponseRepresentable {
    let groups = try tiluser.groups()
    return try JSON (node: groups.makeNode())
  }

}

extension Request {
  func tiluser() throws -> TILUser {
    guard let json = json else { throw Abort.badRequest }
    return try TILUser(node: json)
  }
}
