import Vapor

final class Acronym: Model {

  var id: Node?
  var exists: Bool = false

  var short: String
  var long: String
  var tiluserId: Node?
  
  init(short: String, long: String, tiluserId: Node? = nil) {
    self.id = nil
    self.short = short
    self.long = long
    self.tiluserId = tiluserId
  }
  
  init(node: Node, in context: Context) throws {
    id = try node.extract("id")
    short = try node.extract("short")
    long = try node.extract("long")
    tiluserId = try node.extract("tiluser_id")
  }
  
  func makeNode(context: Context) throws -> Node {
    return try Node(node: [
      "id": id,
      "short": short,
      "long": long,
      "tiluser_id": tiluserId
    ])
  }
  
  static func prepare(_ database: Database) throws {
    try database.create("acronyms") { users in
      users.id()
      users.string("short")
      users.string("long")
      users.parent(TILUser.self, optional: false)
    }
  }
  
  static func revert(_ database: Database) throws {
    try database.delete("acronyms")
  }

}

extension Acronym {
  func tiluser() throws -> TILUser? {
    return try parent(tiluserId, nil, TILUser.self).get()
  }
}



