import Vapor
import Fluent

final class Group: Model {
  
  var exists: Bool = false
  var id: Node?
  var name: Valid<NameValidator>
  
  init(name: String, email: String, password: String) throws {
    self.name = try name.validated()
  }
    
  init(node: Node, in context: Context) throws {
    id = node["id"]
    let nameString = try node.extract("name") as String
    name = try nameString.validated()
  }
  
  func makeNode(context: Context) throws -> Node {
    return try Node(node: [
      "id": id,
      "name": name.value,
      ])
  }
  
  static func prepare(_ database: Database) throws {
    try database.create("groups") { users in
      users.id()
      users.string("name")
    }
  }
  
  static func revert(_ database: Database) throws {
    try database.delete("groups")
  }
}

extension Group {
  func tilusers() throws -> [TILUser] {
    let tilusers: Siblings<TILUser> = try siblings()
    return try tilusers.all()
  }
}
