import Vapor
import Fluent

final class TILUser: Model {
  
  var exists: Bool = false
  var id: Node?
  var name: Valid<NameValidator>
  var email: Valid<EmailValidator>
  var password: Valid<PasswordValidator>
  
  init(name: String, email: String, password: String) throws {
    self.name = try name.validated()
    self.email = try email.validated()
    self.password = try password.validated()
  }
    
  init(node: Node, in context: Context) throws {
    id = node["id"]
    let nameString = try node.extract("name") as String
    name = try nameString.validated()
    let emailString = try node.extract("email") as String
    email = try emailString.validated()
    let passwordString = try node.extract("password") as String
    password = try passwordString.validated()
  }
  
  func makeNode(context: Context) throws -> Node {
    return try Node(node: [
      "id": id,
      "name": name.value,
      "email": email.value,
      "password": password.value,
      ])
  }
  
  static func prepare(_ database: Database) throws {
    try database.create("tilusers") { users in
      users.id()
      users.string("name")
      users.string("email")
      users.string("password")
    }
  }
  
  static func revert(_ database: Database) throws {
    try database.delete("tilusers")
  }
}

extension TILUser {
  func acronyms() throws -> [Acronym] {
    return try children(nil, Acronym.self).all()
  }
  
  func groups() throws -> [Group] {
    let groups: Siblings<Group> = try siblings()
    return try groups.all()
  }
  
  
}
